#include <stdio.h>
#include <time.h>
#include <stdlib.h>

#include "Wave.h"

int main(int argc, char** argv)
{
	if(argc != 3){
		// usage
		printf("usage: %s [input(.wav)] [output(.wav)]\n", argv[0]);
		printf("The same filename for input and output is not acceptable.\n");
		return 0;
	}
	
	int i, j;

	CWave waveIn, waveOut;
	CWave::tOutputWaveFormat fmt;
	fmt.samplePerSec = 44100;
	fmt.bytePerSample = 2;
	fmt.channel = 1;
	
	const int bufSize = 10240;
	short* buf = new short[bufSize]; // 10k sample = 1/4.4 sec.
	int ret = 0;
	if((ret = waveIn.OpenWave(argv[1])) > 0)
	{
		if(waveOut.OutputWave(argv[2], &fmt) == 0){
			int sampleLength = waveIn.GetSampleLength();
			printf("Input Length = %d samples\n", sampleLength);
			
			int chunkListSize = sampleLength / bufSize + 1;
			char* chunkValidList = new char[chunkListSize];
			for(i=0; i<chunkListSize; i++) chunkValidList[i] = 0;
			
			waveIn.SetTopPage();
			for(i=0; i<sampleLength / bufSize; i++){
				// ...
				waveIn.LoadWaveShort(buf, bufSize, 0, 0); // Read wave file to monoral buf
				if(i<1) continue; // cut out first 1/4 sec.

				// Get max value
				short absMax = 0;
				for(j=0; j<bufSize; j++){
					// valid / invalid check
					if(abs(buf[j]) > 10000) {
						chunkValidList[i] = 1;
						break;
					}
				}
				
				// debug
				if(chunkValidList[i]){
					printf("%d: Valid\n", i);
				}else{
					printf("%d: Invalid\n", i);
				}
			}
			
			// output valid frames with wave shaping
			int firstValidChunk = -1, lastValidChunk = -1;
			for(i=0; i<sampleLength / bufSize; i++){
				if(chunkValidList[i] && firstValidChunk >= 0) lastValidChunk = i;
				if(chunkValidList[i] && firstValidChunk == -1) lastValidChunk = firstValidChunk = i;
			}
			waveIn.SetTopPage();
			const short threshold = +8000;
			short outputVal = 0;
			for(i=0; i<sampleLength / bufSize; i++){
				waveIn.LoadWaveShort(buf, bufSize, 0, 0); // Read wave file to monoral buf
				if(i >= firstValidChunk && i <= lastValidChunk){
					int j;
					for(j=0; j<bufSize; j++){
						if(outputVal == 0 && buf[j] > threshold){
							outputVal = +30000;
						}else if(outputVal > 0 && buf[j] < -threshold){
							outputVal = -outputVal;
						}else if(outputVal < 0 && buf[j] > threshold){
							outputVal = -outputVal;
						}
						// Note that NanoPi NEO 2 line out phase is inversed
						buf[j] = -outputVal;
					}
					waveOut.OutputWaveFrameByMono(buf, bufSize);
				}
			}
			// restore DC level smoothly
			{
				for(i=0; i<bufSize; i++){
					double w = (bufSize-i) / (double)bufSize;
					buf[i] = outputVal * w;
				}
				waveOut.OutputWaveFrameByMono(buf, bufSize);
			}
			
			delete [] chunkValidList;
			
			waveOut.OutputWaveFinalize();
		}
		else
		{
			printf("Error: Output file can't open\n");
		}
	}
	else
	{
		printf("Error: Input file can't open [%d]\n", ret);
	}
	
	delete [] buf;
	
	return 0;
}
